@@ -5,7 +5,7 @@ use warnings;
 use vars qw($TODO $Level $using_open);
 require "test.pl";
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 # now export checkOptree, and those test.pl functions used by tests
 our @EXPORT = qw( checkOptree plan skip skip_all pass is like unlike
@@ -51,15 +51,14 @@ various modes.
     prog   => 'sort @a',	# run in subprocess, aka -MO=Concise
     bcopts => '-exec',		# $opt or \@opts, passed to BC::compile
 
-    errs   => 'Useless variable "@main::a" .*'	# str, regex, [str+] [regex+],
+    errs   => 'Name "main::a" used only once: possible typo at -e line 1.',
+				# str, regex, [str+] [regex+],
 
     # various test options
     # errs   => '.*',		# match against any emitted errs, -w warnings
     # skip => 1,		# skips test
     # todo => 'excuse',		# anticipated failures
     # fail => 1			# force fail (by redirecting result)
-    # retry => 1		# retry on test failure
-    # debug => 1,		# use re 'debug' for retried failures !!
 
     # the 'golden-sample's, (must provide both)
 
@@ -236,16 +235,6 @@ invokes todo('reason')
 For code arguments, this option causes getRendering to redirect the
 rendering operation to STDERR, which causes the regex match to fail.
 
-=head2 retry => 1
-
-If retry is set, and a test fails, it is run a second time, possibly
-with regex debug.
-
-=head2 debug => 1
-
-If a failure is retried, this turns on eval "use re 'debug'", thus
-turning on regex debug.  It's quite verbose, and not hugely helpful.
-
 =head2 noanchors => 1
 
 If set, this relaxes the regex check, which is normally pretty strict.
@@ -274,17 +263,6 @@ checkErrs() is a getRendering helper that verifies that expected errs
 against those found when rendering the code on the platform.  It is
 run after rendering, and before mkCheckRex.
 
-Errors can be reported 3 different ways; diag, fail, print.
-
-  diag - uses test.pl _diag()
-  fail - causes double-testing
-  print-.no # in front of the output (may mess up test harnesses)
-
-The 3 ways are selectable at runtimve via cmdline-arg:
-report={diag,fail,print}.  
-
-
-
 =cut
 
 use Config;
@@ -312,8 +290,6 @@ sub import {
 our %gOpts = 	# values are replaced at runtime !!
     (
      # scalar values are help string
-     retry	=> 'retry failures after turning on re debug',
-     debug	=> 'turn on re debug for those retries',
      selftest	=> 'self-tests mkCheckRex vs the reference rendering',
 
      fail	=> 'force all test to fail, print to stdout',
@@ -325,10 +301,6 @@ our %gOpts = 	# values are replaced at runtime !!
      help	=> [0, 'provides help and exits', 0],
      testmode	=> [qw/ native cross both /],
 
-     # reporting mode for rendering errs
-     report	=> [qw/ diag fail print /],
-     errcont	=> [1, 'if 1, tests match even if report is fail', 0],
-
      # fixup for VMS, cygwin, which don't have stderr b4 stdout
      rxnoorder	=> [1, 'if 1, dont req match on -e lines, and -banner',0],
      strip	=> [1, 'if 1, catch errs and remove from renderings',0],
@@ -466,19 +438,8 @@ sub newTestCases {
 	    $tc->{$k} = $gOpts{$k} unless defined $tc->{$k};
 	}
     }
-    # transform errs to self-hash for efficient set-math
     if ($tc->{errs}) {
-	if (not ref $tc->{errs}) {
-	    $tc->{errs} = { $tc->{errs} => 1};
-	}
-	elsif (ref $tc->{errs} eq 'ARRAY') {
-	    my %errs;
-	    @errs{@{$tc->{errs}}} = (1) x @{$tc->{errs}};
-	    $tc->{errs} = \%errs;
-	}
-	elsif (ref $tc->{errs} eq 'Regexp') {
-	    warn "regexp err matching not yet implemented";
-	}
+	$tc->{errs} = [$tc->{errs}] unless ref $tc->{errs} eq 'ARRAY';
     }
     return $tc;
 }
@@ -573,43 +534,36 @@ sub checkErrs {
     # check rendering errs against expected errors, reduce and report
     my $tc = shift;
 
-    # check for agreement, by hash (order less important)
-    my (%goterrs, @got);
-    $tc->{goterrs} ||= [];
-    @goterrs{@{$tc->{goterrs}}} = (1) x scalar @{$tc->{goterrs}};
-    
-    foreach my $k (keys %{$tc->{errs}}) {
-	if (@got = grep /^$k$/, keys %goterrs) {
-	    delete $tc->{errs}{$k};
-	    delete $goterrs{$_} foreach @got;
+    # check for agreement (order not important)
+    my (%goterrs, @missed);
+    @goterrs{@{$tc->{goterrs}}} = (1) x scalar @{$tc->{goterrs}}
+	if $tc->{goterrs};
+
+    foreach my $want (@{$tc->{errs}}) {
+	if (ref $want) {
+	    my $seen;
+	    foreach my $k (keys %goterrs) {
+		next unless $k =~ $want;
+		delete $goterrs{$k};
+		++$seen;
+	    }
+	    push @missed, $want unless $seen;
+	} else {
+	    push @missed, $want unless defined delete $goterrs{$want};
 	}
     }
-    $tc->{goterrs} = \%goterrs;
 
-    # relook at altered
-    if (%{$tc->{errs}} or %{$tc->{goterrs}}) {
-	$tc->diag_or_fail();
-    }
-    fail("FORCED: $tc->{name}:\n") if $gOpts{fail}; # silly ?
-}
-
-sub diag_or_fail {
-    # help checkErrs
-    my $tc = shift;
-
-    my @lines;
-    push @lines, "got unexpected:", sort keys %{$tc->{goterrs}} if %{$tc->{goterrs}};
-    push @lines, "missed expected:", sort keys %{$tc->{errs}}   if %{$tc->{errs}};
-
-    if (@lines) {
-	unshift @lines, $tc->{name};
-	my $report = join("\n", @lines);
-
-	if    ($gOpts{report} eq 'diag')	{ _diag ($report) }
-	elsif ($gOpts{report} eq 'fail')	{ fail  ($report) }
-	else					{ print ($report) }
-	next unless $gOpts{errcont}; # skip block
+    @missed = sort @missed;
+    my @got = sort keys %goterrs;
+
+    if (@{$tc->{errs}}) {
+	is(@missed + @got, 0, "Only got expected errors for $tc->{name}")
+    } else {
+	# @missed must be 0 here.
+	is(scalar @got, 0, "Got no errors for $tc->{name}")
     }
+    _diag(join "\n", "got unexpected:", @got) if @got;
+    _diag(join "\n", "missed expected:", @missed) if @missed;
 }
 
 =head1 mkCheckRex ($tc)
@@ -657,7 +611,6 @@ sub mkCheckRex {
     # converts expected text into Regexp which should match against
     # unaltered version.  also adjusts threaded => non-threaded
     my ($tc, $want) = @_;
-    eval "no re 'debug'";
 
     my $str = $tc->{expect} || $tc->{expect_nt};	# standard bias
     $str = $tc->{$want} if $want && $tc->{$want};	# stated pref
@@ -748,7 +701,7 @@ sub mkCheckRex {
     $str =~ s/leavesub \[\d\]/leavesub [\\d]/msg;	# for -terse
     #$str =~ s/(\s*)\n/\n/msg;				# trailing spaces
     
-    croak "no reftext found for $want: $tc->{name}"
+    croak "whitespace only reftext found for '$want': $tc->{name}"
 	unless $str =~ /\w+/; # fail unless a real test
     
     # $str = '.*'	if 1;	# sanity test
@@ -776,28 +729,13 @@ sub mylike {
     my $cmnt	= $tc->{name};
     my $cross	= $tc->{cross};
 
-    my $msgs	= $tc->{msgs};
-    my $retry	= $tc->{retry}; # || $gopts{retry};
-    my $debug	= $tc->{debug}; #|| $gopts{retrydbg};
-
     # bad is anticipated failure
-    my $bad = (0 or ( $cross && $tc->{crossfail})
-	       or (!$cross && $tc->{fail})
-	       or 0); # no undefs !
+    my $bad = ($cross && $tc->{crossfail}) || (!$cross && $tc->{fail});
 
-    # same as A ^ B, but B has side effects
-    my $ok = ( $bad  &&  unlike ($got, $want, $cmnt, @$msgs)
-	       or !$bad && like ($got, $want, $cmnt, @$msgs));
+    my $ok = $bad ? unlike ($got, $want, $cmnt) : like ($got, $want, $cmnt);
 
     reduceDiffs ($tc) if not $ok;
 
-    if (not $ok and $retry) {
-	# redo, perhaps with use re debug - NOT ROBUST
-	eval "use re 'debug'" if $debug;
-	$ok = ( $bad  &&  unlike ($got, $want, "(RETRY) $cmnt", @$msgs)
-		or !$bad && like ($got, $want, "(RETRY) $cmnt", @$msgs));
-	eval "no re 'debug'";
-    }
     return $ok;
 }
 
